@@ -80,6 +80,7 @@ struct _GeditAppPrivate
 	GMenuModel        *window_menu;
 	GMenuModel        *notebook_menu;
 	GMenuModel        *tab_width_menu;
+	GMenuModel        *line_col_menu;
 
 	PeasExtensionSet  *extensions;
 	GNetworkMonitor   *monitor;
@@ -501,6 +502,7 @@ gedit_app_startup (GApplication *application)
 
 		app->priv->notebook_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "notebook_menu"));
 		app->priv->tab_width_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "tab_width_menu"));
+		app->priv->line_col_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "line_col_menu"));
 	}
 
 	g_object_unref (builder);
@@ -1521,6 +1523,26 @@ find_extension_point_section (GMenuModel  *model,
 		{
 			section = g_menu_model_get_item_link (model, i, G_MENU_LINK_SECTION);
 		}
+		/* unamed sections are to be checked recursively */
+		else if (id == NULL)
+		{
+			GMenuModel *subsection;
+			GMenuModel *submenu;
+			gint j, j_items;
+
+			subsection = g_menu_model_get_item_link (model, i, G_MENU_LINK_SECTION);
+
+			j_items = g_menu_model_get_n_items (subsection);
+
+			for (j = 0; j < j_items && !section; j++)
+			{
+				submenu = g_menu_model_get_item_link (subsection, j, G_MENU_LINK_SUBMENU);
+				if (submenu)
+				{
+					section = find_extension_point_section (submenu, extension_point);
+				}
+			}
+		}
 
 		g_free (id);
 	}
@@ -1656,6 +1678,15 @@ _gedit_app_get_tab_width_menu (GeditApp *app)
 	return app->priv->tab_width_menu;
 }
 
+GMenuModel *
+_gedit_app_get_line_col_menu (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	return app->priv->line_col_menu;
+}
+
+
 GeditMenuExtension *
 _gedit_app_extend_menu (GeditApp    *app,
                        const gchar *extension_point)
