@@ -959,10 +959,8 @@ gedit_no_backup_saving_error_info_bar_new (GFile        *location,
 
 	g_return_val_if_fail (G_IS_FILE (location), NULL);
 	g_return_val_if_fail (error != NULL, NULL);
-	g_return_val_if_fail (((error->domain == GEDIT_DOCUMENT_ERROR &&
-			        error->code == GEDIT_DOCUMENT_ERROR_CANT_CREATE_BACKUP) ||
-			       (error->domain == G_IO_ERROR &&
-			        error->code == G_IO_ERROR_CANT_CREATE_BACKUP)), NULL);
+	g_return_val_if_fail (error->domain == G_IO_ERROR &&
+			      error->code == G_IO_ERROR_CANT_CREATE_BACKUP, NULL);
 
 	full_formatted_uri = g_file_get_parse_name (location);
 
